#!/usr/bin/ruby -w
# Encoding: UTF-8
# frozen_string_literal: true
# =========================================================================== #
# === ShowRdocDocu
#
# Use this class to show rdoc documentation.
#
# It will be stored in @result.
#
# Usage example:
#
#   require 'show_rdoc_docu'
#   ShowRdocDocu.new(ARGV)
#
# =========================================================================== #
# require 'show_rdoc_docu'
# =========================================================================== #
require 'rdoc'
require 'stringio'
require 'show_rdoc_docu/version/version.rb'

class ShowRdocDocu # === ShowRdocDocu

  begin
    require 'colours'
    include Colours
  rescue LoadError; end

  begin
    require 'opn'
  rescue LoadError; end

  # ========================================================================= #
  # === initialize
  # ========================================================================= #
  def initialize(
      i           = nil,
      format_type = '--format=markdown',
      run_already = true
    )
    reset
    set_input(i)
    @format_type = format_type # << '--format=ansi' # => ["-T", "--format=ansi"]
    run if run_already
  end

  # ========================================================================= #
  # === reset                                                     (reset tag)
  # ========================================================================= #
  def reset
    @found_a_result = true
    @be_verbose     = true
  end

  # ========================================================================= #
  # === be_verbose?
  # ========================================================================= #
  def be_verbose?
    @be_verbose
  end

  # ========================================================================= #
  # === found_a_result?
  # ========================================================================= #
  def found_a_result?
    @found_a_result
  end; alias found_result? found_a_result? # === found_result?

  # ========================================================================= #
  # === set_input
  # ========================================================================= #
  def set_input(i = nil)
    i = 'Array#push' if i.nil?
    i = 'Array#push' if i.empty?
    i = i.first if i.is_a? Array
    @input = i
  end

  # ========================================================================= #
  # === restore_stdout
  # ========================================================================= #
  def restore_stdout
    $stdout = STDOUT
  end

  # ========================================================================= #
  # === show_result
  #
  # Simply output the result.
  # ========================================================================= #
  def show_result
    e result # This is the string output.
  end; alias report        show_result # === report
       alias report_result show_result # === report_result

  # ========================================================================= #
  # === result?
  # ========================================================================= #
  def result?
    @result
  end; alias result result? # === result

  # ========================================================================= #
  # === result=
  # ========================================================================= #
  def result=(i)
    @result = i
  end

  # ========================================================================= #
  # === run                                                         (run tag)
  # ========================================================================= #
  def run
    _ = %w( -T )
    _ << @format_type
    # ======================================================================= #
    # Next we delegate into RDoc::RI::Driver.process_args 
    # ======================================================================= #
    hash = RDoc::RI::Driver.process_args(_) # This hash can be passed to Driver.new
    rdoc_driver = RDoc::RI::Driver.new(hash)
    rdoc_driver.use_stdout = true
    @result = ''.dup # We push the output into this instance variable.
    $stdout = StringIO.new(@result) # Snatch all std-output.
    begin # Must rescue here.
      # @input is a String such as: "String#[]"
      _dataset = rdoc_driver.display_method(@input) # This will return a Fixnum.
    rescue RDoc::RI::Driver::NotFoundError
      if be_verbose?
        opn; e 'We did not find anything about `'+simp(@input)+'`.'
      end
      @found_a_result = false
    end
    restore_stdout # And restore stdout again.
  end

end

if __FILE__ == $PROGRAM_NAME
  _ = ShowRdocDocu.new(ARGV)
  _.show_result
end # showrdocdocu Dir.chdir